@file:OptIn(ExperimentalMaterial3Api::class)

package com.jerboa.ui.components.home

import androidx.compose.animation.AnimatedVisibility
import androidx.compose.animation.expandVertically
import androidx.compose.animation.shrinkVertically
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.foundation.lazy.rememberLazyListState
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.outlined.Add
import androidx.compose.material.icons.outlined.Bookmarks
import androidx.compose.material.icons.outlined.Close
import androidx.compose.material.icons.outlined.Email
import androidx.compose.material.icons.outlined.ExpandLess
import androidx.compose.material.icons.outlined.ExpandMore
import androidx.compose.material.icons.outlined.FilterList
import androidx.compose.material.icons.outlined.Info
import androidx.compose.material.icons.outlined.LocationCity
import androidx.compose.material.icons.outlined.Login
import androidx.compose.material.icons.outlined.Menu
import androidx.compose.material.icons.outlined.MoreVert
import androidx.compose.material.icons.outlined.Person
import androidx.compose.material.icons.outlined.Public
import androidx.compose.material.icons.outlined.Refresh
import androidx.compose.material.icons.outlined.Settings
import androidx.compose.material.icons.outlined.Sort
import androidx.compose.material3.AlertDialog
import androidx.compose.material3.Divider
import androidx.compose.material3.DrawerState
import androidx.compose.material3.DrawerValue
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.Icon
import androidx.compose.material3.IconButton
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Text
import androidx.compose.material3.TopAppBar
import androidx.compose.material3.TopAppBarDefaults
import androidx.compose.material3.TopAppBarScrollBehavior
import androidx.compose.material3.rememberDrawerState
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.rememberCoroutineScope
import androidx.compose.runtime.saveable.rememberSaveable
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.tooling.preview.Preview
import androidx.navigation.NavController
import androidx.navigation.compose.rememberNavController
import com.jerboa.datatypes.CommunitySafe
import com.jerboa.datatypes.ListingType
import com.jerboa.datatypes.PersonSafe
import com.jerboa.datatypes.SortType
import com.jerboa.datatypes.api.GetUnreadCountResponse
import com.jerboa.datatypes.api.MyUserInfo
import com.jerboa.datatypes.samplePersonSafe
import com.jerboa.db.Account
import com.jerboa.db.AccountViewModel
import com.jerboa.ui.components.common.IconAndTextDrawerItem
import com.jerboa.ui.components.common.LargerCircularIcon
import com.jerboa.ui.components.common.ListingTypeOptionsDialog
import com.jerboa.ui.components.common.PictrsBannerImage
import com.jerboa.ui.components.common.SortOptionsDialog
import com.jerboa.ui.components.common.SortTopOptionsDialog
import com.jerboa.ui.components.common.simpleVerticalScrollbar
import com.jerboa.ui.components.community.CommunityLinkLarger
import com.jerboa.ui.components.person.PersonName
import com.jerboa.ui.theme.DRAWER_BANNER_SIZE
import com.jerboa.ui.theme.LARGE_PADDING
import com.jerboa.ui.theme.SMALL_PADDING
import com.jerboa.ui.theme.XL_PADDING
import com.jerboa.ui.theme.muted
import com.jerboa.unreadCountTotal
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.launch

@Composable
fun Drawer(
    navController: NavController = rememberNavController(),
    accountViewModel: AccountViewModel,
    onSwitchAccountClick: (account: Account) -> Unit,
    onSignOutClick: () -> Unit,
    onClickListingType: (ListingType) -> Unit,
    myUserInfo: MyUserInfo?,
    onCommunityClick: (community: CommunitySafe) -> Unit,
    onClickProfile: () -> Unit,
    onClickInbox: () -> Unit,
    onClickSaved: () -> Unit,
    onClickSettings: () -> Unit,
    unreadCounts: GetUnreadCountResponse?
) {
    var showAccountAddMode by rememberSaveable { mutableStateOf(false) }

    DrawerHeader(
        myPerson = myUserInfo?.local_user_view?.person,
        showAccountAddMode = showAccountAddMode,
        onClickShowAccountAddMode = { showAccountAddMode = !showAccountAddMode }
    )
    Divider()
    // Drawer items
    DrawerContent(
        accountViewModel = accountViewModel,
        unreadCounts = unreadCounts,
        myUserInfo = myUserInfo,
        showAccountAddMode = showAccountAddMode,
        navController = navController,
        onSwitchAccountClick = onSwitchAccountClick,
        onSignOutClick = onSignOutClick,
        onClickListingType = onClickListingType,
        onCommunityClick = onCommunityClick,
        onClickProfile = onClickProfile,
        onClickInbox = onClickInbox,
        onClickSaved = onClickSaved,
        onClickSettings = onClickSettings
    )
}

@Composable
fun DrawerContent(
    showAccountAddMode: Boolean,
    navController: NavController,
    accountViewModel: AccountViewModel,
    onSwitchAccountClick: (account: Account) -> Unit,
    onSignOutClick: () -> Unit,
    onClickListingType: (ListingType) -> Unit,
    onCommunityClick: (community: CommunitySafe) -> Unit,
    onClickProfile: () -> Unit,
    onClickInbox: () -> Unit,
    onClickSaved: () -> Unit,
    onClickSettings: () -> Unit,
    myUserInfo: MyUserInfo?,
    unreadCounts: GetUnreadCountResponse?
) {
    AnimatedVisibility(
        visible = showAccountAddMode,
        enter = expandVertically(),
        exit = shrinkVertically()
    ) {
        DrawerAddAccountMode(
            accountViewModel = accountViewModel,
            navController = navController,
            onSwitchAccountClick = onSwitchAccountClick,
            onSignOutClick = onSignOutClick
        )
    }

    if (!showAccountAddMode) {
        DrawerItemsMain(
            myUserInfo = myUserInfo,
            onClickListingType = onClickListingType,
            onCommunityClick = onCommunityClick,
            onClickProfile = onClickProfile,
            onClickInbox = onClickInbox,
            onClickSaved = onClickSaved,
            unreadCounts = unreadCounts,
            onClickSettings = onClickSettings
        )
    }
}

@Composable
fun DrawerItemsMain(
    myUserInfo: MyUserInfo?,
    onClickSaved: () -> Unit,
    onClickProfile: () -> Unit,
    onClickInbox: () -> Unit,
    onClickSettings: () -> Unit,
    onClickListingType: (ListingType) -> Unit,
    onCommunityClick: (community: CommunitySafe) -> Unit,
    unreadCounts: GetUnreadCountResponse? = null
) {
    val listState = rememberLazyListState()

    val totalUnreads = unreadCounts?.let { unreadCountTotal(it) }
    val follows = myUserInfo?.follows

    LazyColumn(
        state = listState,
        modifier = Modifier.simpleVerticalScrollbar(listState)
    ) {
        if (!follows.isNullOrEmpty()) {
            item {
                IconAndTextDrawerItem(
                    text = "Subscribed",
                    icon = Icons.Outlined.Bookmarks,
                    onClick = { onClickListingType(ListingType.Subscribed) }
                )
            }
        }
        item {
            IconAndTextDrawerItem(
                text = "Local",
                icon = Icons.Outlined.LocationCity,
                onClick = { onClickListingType(ListingType.Local) }
            )
        }
        item {
            IconAndTextDrawerItem(
                text = "All",
                icon = Icons.Outlined.Public,
                onClick = { onClickListingType(ListingType.All) }
            )
        }
        item {
            myUserInfo?.also {
                IconAndTextDrawerItem(
                    text = "Saved",
                    icon = Icons.Outlined.Bookmarks,
                    onClick = onClickSaved
                )
            }
        }
        item {
            myUserInfo?.also {
                Divider()
            }
        }
        item {
            myUserInfo?.also {
                IconAndTextDrawerItem(
                    text = "Profile",
                    icon = Icons.Outlined.Person,
                    onClick = onClickProfile
                )
            }
        }
        item {
            myUserInfo?.also {
                IconAndTextDrawerItem(
                    text = "Inbox",
                    icon = Icons.Outlined.Email,
                    onClick = onClickInbox,
                    iconBadgeCount = totalUnreads
                )
            }
        }
        item {
            myUserInfo?.also {
                IconAndTextDrawerItem(
                    text = "Settings",
                    icon = Icons.Outlined.Settings,
                    onClick = onClickSettings
                )
            }
        }
        item {
            myUserInfo?.also {
                Divider()
            }
        }

        follows?.also { follows ->
            item {
                Text(
                    text = "Subscriptions",
                    modifier = Modifier.padding(LARGE_PADDING),
                    color = MaterialTheme.colorScheme.onBackground.muted
                )
            }
            items(
                follows,
                key = { follow -> follow.community.id }
            ) { follow ->
                CommunityLinkLarger(
                    community = follow.community,
                    onClick = onCommunityClick
                )
            }
        }
    }
}

@Preview
@Composable
fun DrawerItemsMainPreview() {
    DrawerItemsMain(
        myUserInfo = null,
        onClickListingType = {},
        onClickProfile = {},
        onClickInbox = {},
        onCommunityClick = {},
        onClickSaved = {},
        onClickSettings = {}
    )
}

@Composable
fun DrawerAddAccountMode(
    navController: NavController = rememberNavController(),
    accountViewModel: AccountViewModel?,
    onSwitchAccountClick: (account: Account) -> Unit,
    onSignOutClick: () -> Unit
) {
    val accountsWithoutCurrent = accountViewModel?.allAccounts?.value?.toMutableList()
    val currentAccount = accountsWithoutCurrent?.firstOrNull { it.current }
    accountsWithoutCurrent?.remove(currentAccount)

    Column {
        IconAndTextDrawerItem(
            text = "Add Account",
            icon = Icons.Outlined.Add,
            onClick = { navController.navigate(route = "login") }
        )
        accountsWithoutCurrent?.forEach {
            IconAndTextDrawerItem(
                text = "Switch to ${it.instance}/${it.name}",
                icon = Icons.Outlined.Login,
                onClick = { onSwitchAccountClick(it) }
            )
        }
        currentAccount?.also {
            IconAndTextDrawerItem(
                text = "Sign Out",
                icon = Icons.Outlined.Close,
                onClick = onSignOutClick
            )
        }
    }
}

@Preview
@Composable
fun DrawerAddAccountModePreview() {
    DrawerAddAccountMode(
        onSignOutClick = {},
        onSwitchAccountClick = {},
        accountViewModel = null
    )
}

@Composable
fun DrawerHeader(
    myPerson: PersonSafe?,
    onClickShowAccountAddMode: () -> Unit,
    showAccountAddMode: Boolean = false
) {
    val sizeMod = Modifier
        .fillMaxWidth()
        .height(DRAWER_BANNER_SIZE)

    Box(
        modifier = sizeMod
            .clickable(onClick = onClickShowAccountAddMode)
    ) {
        myPerson?.banner?.also {
            PictrsBannerImage(
                url = it
            )
        }
        // banner
        Row(
            horizontalArrangement = Arrangement.SpaceBetween,
            verticalAlignment = Alignment.CenterVertically,
            modifier = sizeMod
                .padding(XL_PADDING)
        ) {
            AvatarAndAccountName(myPerson)
            Icon(
                imageVector = if (showAccountAddMode) {
                    Icons.Outlined.ExpandLess
                } else {
                    Icons.Outlined.ExpandMore
                },
                contentDescription = "TODO"
            )
        }
    }
}

@Composable
fun AvatarAndAccountName(myPerson: PersonSafe?) {
    Row(
        verticalAlignment = Alignment.CenterVertically,
        horizontalArrangement = Arrangement.spacedBy(SMALL_PADDING)
    ) {
        myPerson?.avatar?.also {
            LargerCircularIcon(icon = it)
        }
        PersonName(
            person = myPerson,
            color = Color.White
        )
    }
}

@Preview
@Composable
fun DrawerHeaderPreview() {
    DrawerHeader(
        myPerson = samplePersonSafe,
        onClickShowAccountAddMode = {}
    )
}

@Composable
fun HomeHeaderTitle(
    selectedSortType: SortType,
    selectedListingType: ListingType
) {
    Column {
        Text(
            text = selectedListingType.toString(),
            style = MaterialTheme.typography.titleLarge
        )
        Text(
            text = selectedSortType.toString(),
            style = MaterialTheme.typography.titleMedium
        )
    }
}

@Composable
fun HomeHeader(
    scope: CoroutineScope,
    drawerState: DrawerState,
    onClickSortType: (SortType) -> Unit,
    onClickListingType: (ListingType) -> Unit,
    onClickRefresh: () -> Unit,
    selectedSortType: SortType,
    selectedListingType: ListingType,
    navController: NavController,
    scrollBehavior: TopAppBarScrollBehavior
) {
    var showSortOptions by remember { mutableStateOf(false) }
    var showTopOptions by remember { mutableStateOf(false) }
    var showListingTypeOptions by remember { mutableStateOf(false) }
    var showMoreOptions by remember { mutableStateOf(false) }

    if (showSortOptions) {
        SortOptionsDialog(
            selectedSortType = selectedSortType,
            onDismissRequest = { showSortOptions = false },
            onClickSortType = {
                showSortOptions = false
                onClickSortType(it)
            },
            onClickSortTopOptions = {
                showSortOptions = false
                showTopOptions = !showTopOptions
            }
        )
    }

    if (showTopOptions) {
        SortTopOptionsDialog(
            selectedSortType = selectedSortType,
            onDismissRequest = { showTopOptions = false },
            onClickSortType = {
                showTopOptions = false
                onClickSortType(it)
            }
        )
    }

    if (showListingTypeOptions) {
        ListingTypeOptionsDialog(
            selectedListingType = selectedListingType,
            onDismissRequest = { showListingTypeOptions = false },
            onClickListingType = {
                showListingTypeOptions = false
                onClickListingType(it)
            }
        )
    }

    if (showMoreOptions) {
        HomeMoreDialog(
            onDismissRequest = { showMoreOptions = false },
            onClickRefresh = onClickRefresh,
            navController = navController
        )
    }
    TopAppBar(
        scrollBehavior = scrollBehavior,
        title = {
            HomeHeaderTitle(
                selectedSortType = selectedSortType,
                selectedListingType = selectedListingType
            )
        },
        navigationIcon = {
            IconButton(onClick = {
                scope.launch {
                    drawerState.open()
                }
            }) {
                Icon(
                    Icons.Outlined.Menu,
                    contentDescription = "Menu"
                )
            }
        },
        // No Idea why, but the tint for this is muted?
        actions = {
            IconButton(onClick = {
                showListingTypeOptions = !showListingTypeOptions
            }) {
                Icon(
                    Icons.Outlined.FilterList,
                    contentDescription = "TODO"
                )
            }
            IconButton(onClick = {
                showSortOptions = !showSortOptions
            }) {
                Icon(
                    Icons.Outlined.Sort,
                    contentDescription = "TODO"
                )
            }
            IconButton(onClick = {
                showMoreOptions = !showMoreOptions
            }) {
                Icon(
                    Icons.Outlined.MoreVert,
                    contentDescription = "TODO"
                )
            }
        }
    )
}

@Preview
@Composable
fun HomeHeaderPreview() {
    val scope = rememberCoroutineScope()
    val drawerState = rememberDrawerState(DrawerValue.Closed)
    HomeHeader(
        scope,
        drawerState,
        onClickSortType = {},
        onClickListingType = {},
        onClickRefresh = {},
        selectedSortType = SortType.Hot,
        selectedListingType = ListingType.All,
        navController = rememberNavController(),
        scrollBehavior = TopAppBarDefaults.enterAlwaysScrollBehavior()
    )
}

@Composable
fun HomeMoreDialog(
    onDismissRequest: () -> Unit,
    navController: NavController,
    onClickRefresh: () -> Unit
) {
    AlertDialog(
        onDismissRequest = onDismissRequest,
        text = {
            Column {
                IconAndTextDrawerItem(
                    text = "Refresh",
                    icon = Icons.Outlined.Refresh,
                    onClick = {
                        onDismissRequest()
                        onClickRefresh()
                    }
                )
                IconAndTextDrawerItem(
                    text = "Site Info",
                    icon = Icons.Outlined.Info,
                    onClick = {
                        navController.navigate("siteSidebar")
                        onDismissRequest()
                    }
                )
            }
        },
        confirmButton = {}
    )
}
